
const CRC_NAME			= "CRC-32-STMF4";
const POLYNOMIAL		= 0x04C11DB7;
const INITIAL_REMAINDER	= 0xFFFFFFFF;
const FINAL_XOR_VALUE	= 0x00000000;
const REFLECT_DATA		= false;
const REFLECT_REMAINDER	= false;
const CHECK_VALUE		= 0x00;

const SIZE_OF_CRCVAL = 4;
const WIDTH  =  (8 * SIZE_OF_CRCVAL);
const TOPBIT =  (1 << (WIDTH - 1));


var  crcTable = [];
/*********************************************************************
 *
 * Function:    crcInit()
 * 
 * Description: Populate the partial CRC lookup table.
 *
 * Notes:		This function must be rerun any time the CRC standard
 *				is changed.  If desired, it can be run "offline" and
 *				the table results stored in an embedded system's ROM.
 *
 * Returns:		None defined.
 *
 *********************************************************************/

function crcInit()
{
    var	 remainder;
    var	 dividend;
    var  bit;


    /*
     * Compute the remainder of each possible dividend.
     */
    for (dividend = 0; dividend < 256; ++dividend)
    {
        /*
         * Start with the dividend followed by zeros.
         */
        remainder = dividend << (WIDTH - 8);

        /*
         * Perform modulo-2 division, a bit at a time.
         */
        for (bit = 8; bit > 0; --bit)
        {
            /*
             * Try to divide the current data bit.
             */			
            if (remainder & TOPBIT)
            {
                remainder = (remainder << 1) ^ POLYNOMIAL;
            }
            else
            {
                remainder = (remainder << 1);
            }
        }

        /*
         * Store the result into the table.
         */
        crcTable[dividend] = remainder;
    }

} 



/*********************************************************************
 *
 * Function:    crcFast()
 * 
 * Description: Compute the CRC of a given message.
 *
 * Notes:		crcInit() must be called first.
 *
 * Returns:		The CRC of the message.
 *
 *********************************************************************/
function  crcFast( message,  nBytes)
{
    var	 remainder = INITIAL_REMAINDER;
    var  data;
    var  byte;

    /*
     * Divide the message by the polynomial, a byte at a time.
     */
    for (var byte = 0; byte < nBytes; ++byte)
    {
        data = (message[byte]) ^ (remainder >>> (WIDTH - 8)); // note triple bit shift operator in Javascript
  		remainder = crcTable[data] ^ (remainder << 8);
    }

    /*
     * The final remainder is the CRC.
     */
    return (remainder ^ FINAL_XOR_VALUE) >>> 0; // force convert to Uint32
}  


function crcFastSTMF4(message, offset,  nBytes)
{	
    if ((nBytes - offset) % 4 != 0)
	{
		debugMessage('crc error', (nBytes - offset));
        return 0;
	}
    var buf = [];
    var i = 0;
    for (i = 0; i < nBytes; i += 4)
    {
    	// reverse for endian correction
        buf[i] = message[offset + i + 3];
        buf[i+1] = message[offset +i + 2];
        buf[i+2] = message[offset +i + 1];
        buf[i+3] = message[offset +i];
    }
    return crcFast(buf , nBytes);
}


